import { AuthProvider, GetListParams, GetOneParams, GetManyParams, GetManyReferenceParams, CreateParams, UpdateParams, UpdateManyParams, DeleteParams, DeleteManyParams, DataProvider } from 'react-admin';
import { AxiosRequestHeaders, AxiosInterceptorOptions } from 'axios';
import { PrismaClient } from '@prisma/client';
import * as ra_core from 'ra-core';

type AuditOptions = {
    model: {
        create: Function;
    };
    authProvider: AuthProvider;
    columns?: {
        id?: string;
        date?: string;
        resource?: string;
        action?: string;
        payload?: string;
        author?: string;
    };
    enabledForAction?: {
        create?: boolean;
        update?: boolean;
        delete?: boolean;
    };
    enabledResources?: string[];
};
declare const defaultAuditOptions: Pick<AuditOptions, "columns" | "enabledForAction">;
type AuditActions = "create" | "update" | "delete";

type Request = GetListRequest | GetOneRequest | GetManyRequest | GetManyReferenceRequest | CreateRequest | UpdateRequest | UpdateManyRequest | DeleteRequest | DeleteManyRequest;
type GetListRequest = {
    body: {
        method: "getList";
        filter: object;
        params: GetListParams;
        resource: string;
        model?: string;
    };
};
type GetOneRequest = {
    body: {
        method: "getOne";
        params: GetOneParams;
        resource: string;
        model?: string;
    };
};
type GetManyRequest = {
    body: {
        method: "getMany";
        params: GetManyParams;
        resource: string;
        model?: string;
    };
};
type GetManyReferenceRequest = {
    body: {
        method: "getManyReference";
        params: GetManyReferenceParams;
        resource: string;
        model?: string;
    };
};
type CreateRequest = {
    body: {
        method: "create";
        params: CreateParams;
        resource: string;
        model?: string;
    };
};
type UpdateRequest = {
    body: {
        method: "update";
        params: UpdateParams;
        resource: string;
        model?: string;
    };
};
type UpdateManyRequest = {
    body: {
        method: "updateMany";
        params: UpdateManyParams;
        resource: string;
        model?: string;
    };
};
type DeleteRequest = {
    body: {
        method: "delete";
        params: DeleteParams;
        resource: string;
        model?: string;
    };
};
type DeleteManyRequest = {
    body: {
        method: "deleteMany";
        params: DeleteManyParams;
        resource: string;
        model?: string;
    };
};
declare type Send<T> = (body: T) => void;
type Response<T extends {
    data: any;
} = any> = {
    json: Send<T>;
};

type CreateArgs = {
    include?: object | null;
    select?: object | null;
};
type CreateOptions<Args extends CreateArgs = CreateArgs> = Args & {
    connect?: {
        [key: string]: string | {
            [key: string]: string | {
                [key: string]: string;
            };
        };
    };
    audit?: AuditOptions;
    debug?: boolean;
};
declare const createHandler: <Args extends CreateArgs>(req: CreateRequest, res: Response, model: {
    create: Function;
}, options?: CreateOptions<Omit<Args, "data">>) => Promise<{
    data: any;
}>;

type AxiosInterceptorFulfilled<V> = ((value: V) => V | Promise<V>) | null;
type AxiosInterceptorError = ((error: any) => any) | null;
declare const dataProvider: (endpoint: string, options?: {
    headers?: AxiosRequestHeaders;
    withCredentials?: boolean;
    resourceToModelMap?: Record<string, string>;
    axiosInterceptors?: {
        response?: {
            onFulfilled?: AxiosInterceptorFulfilled<any>;
            onRejected?: AxiosInterceptorError;
            options?: AxiosInterceptorOptions;
        }[];
        request?: {
            onFulfilled?: AxiosInterceptorFulfilled<any>;
            onRejected?: AxiosInterceptorError;
            options?: AxiosInterceptorOptions;
        }[];
    };
}) => DataProvider;

type DeleteOptions = {
    softDeleteField?: string;
    debug?: boolean;
    audit?: AuditOptions;
};
declare const deleteHandler: <W extends {
    include?: object | null;
    select?: object | null;
    where?: object | null;
}>(req: DeleteRequest, res: Response, model: {
    update: Function;
    delete: Function;
}, options?: DeleteOptions) => Promise<{
    data: any;
}>;

type FilterMode = "insensitive" | "default" | undefined;
type ExtractWhereOptions = {
    filterMode?: FilterMode;
};
declare const extractWhere: (req: GetListRequest | GetManyReferenceRequest, options?: ExtractWhereOptions) => {};

type GetListArgs = {
    include?: object | null;
    select?: object | null;
    where?: object | null;
    orderBy?: object | null;
};
type GetListOptions<Args extends GetListArgs = GetListArgs> = Args & {
    noNullsOnSort?: string[];
    debug?: boolean;
    transform?: (data: any) => any;
    filterMode?: FilterMode;
};
declare const getListHandler: <Args extends GetListArgs>(req: GetListRequest, res: Response, model: {
    findMany: Function;
    count: Function;
}, options?: GetListOptions<Args>) => Promise<{
    data: any;
    total: any;
}>;

type UpdateArgs = {
    include?: object | null;
    select?: object | null;
};
type UpdateOptions<Args extends UpdateArgs = UpdateArgs> = Args & {
    debug?: boolean;
    skipFields?: {
        [key: string]: boolean;
    };
    set?: {
        [key: string]: string | {
            [key: string]: string | {
                [key: string]: string;
            };
        };
    };
    allowNestedUpdate?: {
        [key: string]: boolean;
    };
    allowNestedUpsert?: {
        [key: string]: boolean;
    };
    allowJsonUpdate?: {
        [key: string]: boolean;
    };
    audit?: AuditOptions;
};
declare const reduceData: (data: any, options: UpdateOptions) => {};
declare const updateHandler: <Args extends UpdateArgs>(req: UpdateRequest, res: Response, model: {
    update: Function;
}, options?: UpdateOptions<Omit<Args, "data" | "where">>) => Promise<{
    data: any;
}>;

type GetManyArgs = {
    include?: object | null;
    select?: object | null;
};
type GetManyOptions<Args extends GetManyArgs = GetManyArgs> = Args & {
    debug?: boolean;
    transform?: (data: any) => any;
};
declare const getManyHandler: <Args extends GetManyArgs>(req: GetManyRequest, res: Response, model: {
    findMany: Function;
}, options?: GetManyOptions<Args>) => Promise<{
    data: any;
}>;

type GetManyRefernceArgs = {
    include?: object | null;
    select?: object | null;
};
type GetManyReferenceOptions<Args extends GetManyRefernceArgs = GetManyRefernceArgs> = Args & {
    debug?: boolean;
    transform?: (data: any) => any;
    filterMode?: FilterMode;
};
declare const getManyReferenceHandler: <Args extends GetManyRefernceArgs>(req: GetManyReferenceRequest, res: Response, model: {
    findMany: Function;
}, options?: GetManyReferenceOptions<Args>) => Promise<{
    data: any;
    total: any;
}>;

type GetOneArgs = {
    include?: object | null;
    select?: object | null;
};
type GetOneOptions<Args extends GetOneArgs = GetOneArgs> = Args & {
    debug?: boolean;
    transform?: (row: any) => any;
};
declare const getOneHandler: <Args extends GetOneArgs>(req: GetOneRequest, res: Response, model: {
    findUnique: Function;
}, options?: GetOneOptions<Omit<Args, "where">>) => Promise<{
    data: any;
}>;

declare const defaultHandler: (req: Request, res: Response, prisma: PrismaClient, options?: {
    audit?: AuditOptions;
    create?: CreateOptions;
    delete?: DeleteOptions;
    getList?: GetListOptions;
    getMany?: GetManyOptions;
    getManyReference?: GetManyReferenceOptions;
    getOne?: GetOneOptions;
    update?: UpdateOptions;
}) => Promise<{
    data: any;
}>;

type DeleteManyOptions = {
    softDeleteField?: string;
    audit?: AuditOptions;
    debug?: boolean;
};
declare const deleteManyHandler: (req: DeleteManyRequest, res: Response, model: {
    updateMany: Function;
    deleteMany: Function;
}, options?: DeleteManyOptions) => Promise<{
    data: any[];
}>;

declare const extractOrderBy: (req: GetListRequest | GetManyReferenceRequest) => {};

declare const extractSkipTake: (req: GetListRequest | GetManyReferenceRequest) => {
    skip: any;
    take: any;
};

declare const updateManyHandler: <Args extends UpdateArgs>(req: UpdateManyRequest, res: Response, model: {
    updateMany: Function;
}, options?: Omit<UpdateOptions<Args>, "include" | "select">) => Promise<{
    data: ra_core.Identifier[];
}>;

declare const auditHandler: (request: Request, options: AuditOptions, created?: any) => Promise<void>;
declare const createAuditEntry: (options: AuditOptions, request: Request, id: any) => Promise<any>;

interface Permission<T> {
    /**
     * @default 'allow'
     */
    type?: "allow" | "deny";
    /**
     * List of actions to allow/deny for the resource
     * Can be a single action or an array of actions
     */
    action: Action | Action[];
    /**
     * Resource to allow/deny the actions on
     * This is ideally typed but is also of type string for joint resources ex. user.email
     */
    resource: T | "*" | string;
    record?: any;
}
type Action = "*" | "list" | "show" | "create" | "edit" | "delete" | "export";
type Permissions<T> = Permission<T>[];

type ReactAdminFetchActions = "getOne" | "create" | "update" | "getList" | "getMany" | "getManyReference" | "updateMany" | "delete" | "deleteMany";
declare const fetchActionToAction: Record<ReactAdminFetchActions, Action>;
declare const canAccess: ({ action, permissions, resource, record, }: {
    action: Action;
    permissions: Permissions<string>;
    resource: string;
    record?: any;
}) => boolean;

export { Action, AuditActions, AuditOptions, CreateArgs, CreateOptions, CreateRequest, DeleteManyOptions, DeleteManyRequest, DeleteOptions, DeleteRequest, FilterMode, GetListArgs, GetListOptions, GetListRequest, GetManyArgs, GetManyOptions, GetManyReferenceOptions, GetManyReferenceRequest, GetManyRefernceArgs, GetManyRequest, GetOneArgs, GetOneOptions, GetOneRequest, Permission, Permissions, ReactAdminFetchActions, Request, Response, UpdateArgs, UpdateManyRequest, UpdateOptions, UpdateRequest, auditHandler, canAccess, createAuditEntry, createHandler, dataProvider, defaultAuditOptions, defaultHandler, deleteHandler, deleteManyHandler, extractOrderBy, extractSkipTake, extractWhere, fetchActionToAction, getListHandler, getManyHandler, getManyReferenceHandler, getOneHandler, reduceData, updateHandler, updateManyHandler };
